@@ -43,6 +43,7 @@ from heat.engine import parent_rsrc
 from heat.engine import resource
 from heat.engine import resources
 from heat.engine import scheduler
+from heat.engine import status
 from heat.engine import stk_defn
 from heat.engine import sync_point
 from heat.engine import template as tmpl
@@ -1349,7 +1350,7 @@ class Stack(collections.Mapping):
         self.reset_dependencies()
         self._resources = None
 
-        if action is not self.CREATE:
+        if action != self.CREATE:
             self.updated_time = oslo_timeutils.utcnow()
 
         if new_stack is not None:
@@ -1473,26 +1474,33 @@ class Stack(collections.Mapping):
         self.converge_stack(rollback_tmpl, action=self.ROLLBACK)
 
     def _get_best_existing_rsrc_db(self, rsrc_name):
-        candidate = None
         if self.ext_rsrcs_db:
-            for ext_rsrc in self.ext_rsrcs_db.values():
-                if ext_rsrc.name != rsrc_name:
-                    continue
-                if ext_rsrc.current_template_id == self.t.id:
-                    # Rollback where the previous resource still exists
-                    candidate = ext_rsrc
-                    break
-                elif (ext_rsrc.current_template_id ==
-                        self.prev_raw_template_id):
-                    # Current resource is otherwise a good candidate
-                    candidate = ext_rsrc
-                elif candidate is None:
-                    # In multiple concurrent updates, if candidate is not
-                    # found in current/previous template, it could be found
-                    # in old tmpl.
-                    candidate = ext_rsrc
+            def suitability(ext_rsrc):
+                score = 0
 
-        return candidate
+                if ext_rsrc.status == status.ResourceStatus.FAILED:
+                    score -= 30
+                if ext_rsrc.action == status.ResourceStatus.DELETE:
+                    score -= 50
+                if ext_rsrc.replaced_by:
+                    score -= 1
+                if ext_rsrc.current_template_id == self.prev_raw_template_id:
+                    # Current resource
+                    score += 5
+                if ext_rsrc.current_template_id == self.t.id:
+                    # Rolling back to previous resource
+                    score += 10
+
+                return score, ext_rsrc.updated_at
+
+            candidates = sorted((r for r in self.ext_rsrcs_db.values()
+                                 if r.name == rsrc_name),
+                                key=suitability,
+                                reverse=True)
+            if candidates:
+                return candidates[0]
+
+        return None
 
     def _update_or_store_resources(self):
         self.ext_rsrcs_db = self.db_active_resources_get()
@@ -1509,14 +1517,6 @@ class Stack(collections.Mapping):
                 rsrcs[existing_rsrc_db.name] = existing_rsrc_db
         return rsrcs
 
-    def set_resource_deps(self):
-        curr_name_translated_dep = self.dependencies.translate(lambda res:
-                                                               res.id)
-        ext_rsrcs_db = self.db_active_resources_get()
-        for r in self.dependencies:
-            r.requires = list(curr_name_translated_dep.requires(r.id))
-            resource.Resource.set_requires(ext_rsrcs_db[r.id], r.requires)
-
     def _compute_convg_dependencies(self, existing_resources,
                                     current_template_deps, current_resources):
         def make_graph_key(rsrc):
@@ -2249,21 +2249,24 @@ class Stack(collections.Mapping):
         return False
 
     def migrate_to_convergence(self):
-        values = {'current_template_id': self.t.id}
         db_rsrcs = self.db_active_resources_get()
+        res_id_dep = self.dependencies.translate(lambda res: res.id)
+        current_template_id = self.t.id
         if db_rsrcs is not None:
-            for res in db_rsrcs.values():
-                # delete db resources not in current_template_id
-                try:
-                    self.defn.resource_definition(res.name)
-                except KeyError:
+            for db_res in db_rsrcs.values():
+                requires = set(res_id_dep.requires(db_res.id))
+                r = self.resources.get(db_res.name)
+                if r is None:
+                    # delete db resources not in current_template_id
                     LOG.warning("Resource %(res)s not found in template "
                                 "for stack %(st)s, deleting from db.",
-                                {'res': res.name, 'st': self.id})
-                    resource_objects.Resource.delete(self.context, res.id)
+                                {'res': db_res.name, 'st': self.id})
+                    resource_objects.Resource.delete(self.context, db_res.id)
                 else:
-                    res.update_and_save(values=values)
-        self.set_resource_deps()
+                    r.requires = requires
+                    db_res.convert_to_convergence(current_template_id,
+                                                  requires)
+
         self.current_traversal = uuidutils.generate_uuid()
         self.convergence = True
         prev_raw_template_id = self.prev_raw_template_id
